@@ -1,6 +1,6 @@
 // merge.h -- handle section merging for gold  -*- C++ -*-
 
-// Copyright 2006, 2007, 2008, 2010 Free Software Foundation, Inc.
+// Copyright (C) 2006-2015 Free Software Foundation, Inc.
 // Written by Ian Lance Taylor <iant@google.com>.
 
 // This file is part of gold.
@@ -33,8 +33,6 @@
 namespace gold
 {
 
-class Merge_map;
-
 // For each object with merge sections, we store an Object_merge_map.
 // This is used to map locations in input sections to a merged output
 // section.  The output section itself is not recorded here--it can be
@@ -44,9 +42,7 @@ class Object_merge_map
 {
  public:
   Object_merge_map()
-    : first_shnum_(-1U), first_map_(),
-      second_shnum_(-1U), second_map_(),
-      section_merge_maps_()
+    : section_merge_maps_()
   { }
 
   ~Object_merge_map();
@@ -57,8 +53,9 @@ class Object_merge_map
   // discarded.  OUTPUT_OFFSET is relative to the start of the merged
   // data in the output section.
   void
-  add_mapping(const Merge_map*, unsigned int shndx, section_offset_type offset,
-	      section_size_type length, section_offset_type output_offset);
+  add_mapping(const Output_section_data*, unsigned int shndx,
+              section_offset_type offset, section_size_type length,
+              section_offset_type output_offset);
 
   // Get the output offset for an input address.  MERGE_MAP is the map
   // we are looking for, or NULL if we don't care.  The input address
@@ -68,13 +65,12 @@ class Object_merge_map
   // mapping is known, false otherwise.  *OUTPUT_OFFSET is relative to
   // the start of the merged data in the output section.
   bool
-  get_output_offset(const Merge_map*, unsigned int shndx,
+  get_output_offset(unsigned int shndx,
 		    section_offset_type offset,
 		    section_offset_type* output_offset);
 
-  // Return whether this is the merge map for section SHNDX.
-  bool
-  is_merge_section_for(const Merge_map*, unsigned int shndx);
+  const Output_section_data*
+  find_merge_section(unsigned int shndx) const;
 
   // Initialize an mapping from input offsets to output addresses for
   // section SHNDX.  STARTING_ADDRESS is the output address of the
@@ -87,7 +83,6 @@ class Object_merge_map
       Unordered_map<section_offset_type,
 		    typename elfcpp::Elf_types<size>::Elf_Addr>*);
 
- private:
   // Map input section offsets to a length and an output section
   // offset.  An output section offset of -1 means that this part of
   // the input section is being discarded.
@@ -101,17 +96,12 @@ class Object_merge_map
     section_offset_type output_offset;
   };
 
-  // A less-than comparison routine for Input_merge_entry.
-  struct Input_merge_compare
-  {
-    bool
-    operator()(const Input_merge_entry& i1, const Input_merge_entry& i2) const
-    { return i1.input_offset < i2.input_offset; }
-  };
-
   // A list of entries for a particular input section.
   struct Input_merge_map
   {
+    void add_mapping(section_offset_type input_offset, section_size_type length,
+                     section_offset_type output_offset);
+
     typedef std::vector<Input_merge_entry> Entries;
 
     // We store these with the Relobj, and we look them up by input
@@ -133,82 +123,50 @@ class Object_merge_map
     // look up information for a different Merge_map, we report that
     // we don't have it, rather than trying a lookup and returning an
     // answer which will receive the wrong offset.
-    const Merge_map* merge_map;
+    const Output_section_data* output_data;
     // The list of mappings.
     Entries entries;
     // Whether the ENTRIES field is sorted by input_offset.
     bool sorted;
 
     Input_merge_map()
-      : merge_map(NULL), entries(), sorted(true)
+      : output_data(NULL), entries(), sorted(true)
     { }
   };
 
-  // Map input section indices to merge maps.
-  typedef std::map<unsigned int, Input_merge_map*> Section_merge_maps;
-
-  // Return a pointer to the Input_merge_map to use for the input
-  // section SHNDX, or NULL.
-  Input_merge_map*
-  get_input_merge_map(unsigned int shndx);
-
   // Get or make the Input_merge_map to use for the section SHNDX
   // with MERGE_MAP.
   Input_merge_map*
-  get_or_make_input_merge_map(const Merge_map* merge_map, unsigned int shndx);
+  get_or_make_input_merge_map(const Output_section_data* merge_map,
+                              unsigned int shndx);
+
+  private:
+  // A less-than comparison routine for Input_merge_entry.
+  struct Input_merge_compare
+  {
+    bool
+    operator()(const Input_merge_entry& i1, const Input_merge_entry& i2) const
+    { return i1.input_offset < i2.input_offset; }
+  };
+
+  // Map input section indices to merge maps.
+  typedef std::vector<std::pair<unsigned int, Input_merge_map*> >
+      Section_merge_maps;
+
+  // Return a pointer to the Input_merge_map to use for the input
+  // section SHNDX, or NULL.
+  const Input_merge_map*
+  get_input_merge_map(unsigned int shndx) const;
+
+  Input_merge_map *
+  get_input_merge_map(unsigned int shndx) {
+    return const_cast<Input_merge_map *>(static_cast<const Object_merge_map *>(
+                                             this)->get_input_merge_map(shndx));
+  }
 
-  // Any given object file will normally only have a couple of input
-  // sections with mergeable contents.  So we keep the first two input
-  // section numbers inline, and push any further ones into a map.  A
-  // value of -1U in first_shnum_ or second_shnum_ means that we don't
-  // have a corresponding entry.
-  unsigned int first_shnum_;
-  Input_merge_map first_map_;
-  unsigned int second_shnum_;
-  Input_merge_map second_map_;
   Section_merge_maps section_merge_maps_;
 };
 
-// This class manages mappings from input sections to offsets in an
-// output section.  This is used where input sections are merged.  The
-// actual data is stored in fields in Object.
-
-class Merge_map
-{
- public:
-  Merge_map()
-  { }
-
-  // Add a mapping for the bytes from OFFSET to OFFSET + LENGTH in the
-  // input section SHNDX in object OBJECT to OUTPUT_OFFSET in the
-  // output section.  An OUTPUT_OFFSET of -1 means that the bytes are
-  // discarded.  OUTPUT_OFFSET is not the offset from the start of the
-  // output section, it is the offset from the start of the merged
-  // data within the output section.
-  void
-  add_mapping(Relobj* object, unsigned int shndx,
-	      section_offset_type offset, section_size_type length,
-	      section_offset_type output_offset);
-
-  // Return the output offset for an input address.  The input address
-  // is at offset OFFSET in section SHNDX in OBJECT.  This sets
-  // *OUTPUT_OFFSET to the offset in the output section; this will be
-  // -1 if the bytes are not being copied to the output.  This returns
-  // true if the mapping is known, false otherwise.  This returns the
-  // value stored by add_mapping, namely the offset from the start of
-  // the merged data within the output section.
-  bool
-  get_output_offset(const Relobj* object, unsigned int shndx,
-		    section_offset_type offset,
-		    section_offset_type* output_offset) const;
-
-  // Return whether this is the merge mapping for section SHNDX in
-  // OBJECT.  This should return true when get_output_offset would
-  // return true for some input offset.
-  bool
-  is_merge_section_for(const Relobj* object, unsigned int shndx) const;
-};
-
 // A general class for SHF_MERGE data, to hold functions shared by
 // fixed-size constant data and string data.
 
@@ -216,7 +174,7 @@ class Output_merge_base : public Output_section_data
 {
  public:
   Output_merge_base(uint64_t entsize, uint64_t addralign)
-    : Output_section_data(addralign), merge_map_(), entsize_(entsize),
+    : Output_section_data(addralign), entsize_(entsize),
       keeps_input_sections_(false), first_relobj_(NULL), first_shndx_(-1),
       input_sections_()
   { }
@@ -285,21 +243,6 @@ class Output_merge_base : public Output_section_data
 		   section_offset_type offset,
 		   section_offset_type* poutput) const;
 
-  // Return whether this is the merge section for an input section.
-  bool
-  do_is_merge_section_for(const Relobj*, unsigned int shndx) const;
-
-  // Add a mapping from an OFFSET in input section SHNDX in object
-  // OBJECT to an OUTPUT_OFFSET in the output section.  OUTPUT_OFFSET
-  // is the offset from the start of the merged data in the output
-  // section.
-  void
-  add_mapping(Relobj* object, unsigned int shndx, section_offset_type offset,
-	      section_size_type length, section_offset_type output_offset)
-  {
-    this->merge_map_.add_mapping(object, shndx, offset, length, output_offset);
-  }
-
   // This may be overridden by the child class.
   virtual bool
   do_is_string()
@@ -315,9 +258,6 @@ class Output_merge_base : public Output_section_data
   record_input_section(Relobj* relobj, unsigned int shndx);
 
  private:
-  // A mapping from input object/section/offset to offset in output
-  // section.
-  Merge_map merge_map_;
   // The entry size.  For fixed-size constants, this is the size of
   // the constants.  For strings, this is the size of a character.
   uint64_t entsize_;
