@@ -27,8 +27,9 @@
  * SUCH DAMAGE.
  */
 
-#include "libiberty.h"
 #include "gprof.h"
+#include "libiberty.h"
+#include "bfdver.h"
 #include "search_list.h"
 #include "source.h"
 #include "symtab.h"
@@ -46,9 +47,10 @@
 
 static void usage (FILE *, int) ATTRIBUTE_NORETURN;
 
-const char *whoami;
-const char *function_mapping_file;
-const char *a_out_name = A_OUTNAME;
+const char * whoami;
+const char * function_mapping_file;
+static const char * external_symbol_table;
+const char * a_out_name = A_OUTNAME;
 long hz = HZ_WRONG;
 
 /*
@@ -59,7 +61,6 @@ int output_style = 0;
 int output_width = 80;
 bfd_boolean bsd_style_output = FALSE;
 bfd_boolean demangle = TRUE;
-bfd_boolean discard_underscores = TRUE;
 bfd_boolean ignore_direct_calls = FALSE;
 bfd_boolean ignore_static_funcs = FALSE;
 bfd_boolean ignore_zeros = TRUE;
@@ -67,6 +68,7 @@ bfd_boolean line_granularity = FALSE;
 bfd_boolean print_descriptions = TRUE;
 bfd_boolean print_path = FALSE;
 bfd_boolean ignore_non_functions = FALSE;
+bfd_boolean inline_file_names = FALSE;
 File_Format file_format = FF_AUTO;
 
 bfd_boolean first_output = TRUE;
@@ -84,22 +86,22 @@ static char *default_excluded_list[] =
 {
   "_gprof_mcount", "mcount", "_mcount", "__mcount", "__mcount_internal",
   "__mcleanup",
-  "<locore>", "<hicore>",
   0
 };
 
 /* Codes used for the long options with no short synonyms.  150 isn't
    special; it's just an arbitrary non-ASCII char value.  */
 
-#define OPTION_DEMANGLE		(150)
-#define OPTION_NO_DEMANGLE	(OPTION_DEMANGLE + 1)
-#define OPTION_DEMANGLER	(OPTION_NO_DEMANGLE + 1)
+#define OPTION_DEMANGLE			(150)
+#define OPTION_NO_DEMANGLE		(OPTION_DEMANGLE + 1)
+#define OPTION_INLINE_FILE_NAMES	(OPTION_DEMANGLE + 2)
 
 static struct option long_options[] =
 {
   {"line", no_argument, 0, 'l'},
   {"no-static", no_argument, 0, 'a'},
   {"ignore-non-functions", no_argument, 0, 'D'},
+  {"external-symbol-table", required_argument, 0, 'S'},
 
     /* output styles: */
 
@@ -121,9 +123,9 @@ static struct option long_options[] =
   {"all-lines", no_argument, 0, 'x'},
   {"demangle", optional_argument, 0, OPTION_DEMANGLE},
   {"no-demangle", no_argument, 0, OPTION_NO_DEMANGLE},
-  {"demangler", required_argument, NULL, OPTION_DEMANGLER},
   {"directory-path", required_argument, 0, 'I'},
   {"display-unused-functions", no_argument, 0, 'z'},
+  {"inline-file-names", no_argument, 0, OPTION_INLINE_FILE_NAMES},
   {"min-count", required_argument, 0, 'm'},
   {"print-path", no_argument, 0, 'L'},
   {"separate-files", no_argument, 0, 'y'},
@@ -158,22 +160,21 @@ static void
 usage (FILE *stream, int status)
 {
   fprintf (stream, _("\
-Usage: %s [-[abcDhilLsTvwxyz]] [-[ACeEfFJnNOpPqQZ][name]] [-I dirs]\n\
+Usage: %s [-[abcDhilLsTvwxyz]] [-[ACeEfFJnNOpPqSQZ][name]] [-I dirs]\n\
 	[-d[num]] [-k from/to] [-m min-count] [-t table-length]\n\
 	[--[no-]annotated-source[=name]] [--[no-]exec-counts[=name]]\n\
 	[--[no-]flat-profile[=name]] [--[no-]graph[=name]]\n\
 	[--[no-]time=name] [--all-lines] [--brief] [--debug[=level]]\n\
-	[--function-ordering] [--file-ordering]\n\
+	[--function-ordering] [--file-ordering] [--inline-file-names]\n\
 	[--directory-path=dirs] [--display-unused-functions]\n\
 	[--file-format=name] [--file-info] [--help] [--line] [--min-count=n]\n\
 	[--no-static] [--print-path] [--separate-files]\n\
 	[--static-call-graph] [--sum] [--table-length=len] [--traditional]\n\
 	[--version] [--width=n] [--ignore-non-functions]\n\
-	[--demangle[=%s]]\n\
-	[--no-demangle] [--demangler=dso:function]\n\
+	[--demangle[=STYLE]] [--no-demangle] [--external-symbol-table=name] [@FILE]\n\
 	[image-file] [profile-file...]\n"),
-	   whoami, get_demangler_list ());
-  if (status == 0)
+	   whoami);
+  if (REPORT_BUGS_TO[0] && status == 0)
     fprintf (stream, _("Report bugs to %s\n"), REPORT_BUGS_TO);
   done (status);
 }
@@ -185,7 +186,6 @@ main (int argc, char **argv)
   char **sp, *str;
   Sym **cg = 0;
   int ch, user_specified = 0;
-  const char *demangler = NULL, *style = NULL;
 
 #if defined (HAVE_SETLOCALE) && defined (HAVE_LC_MESSAGES)
   setlocale (LC_MESSAGES, "");
@@ -193,14 +193,18 @@ main (int argc, char **argv)
 #if defined (HAVE_SETLOCALE)
   setlocale (LC_CTYPE, "");
 #endif
+#ifdef ENABLE_NLS
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
+#endif
 
   whoami = argv[0];
   xmalloc_set_program_name (whoami);
 
+  expandargv (&argc, &argv);
+
   while ((ch = getopt_long (argc, argv,
-	"aA::bBcCd::De:E:f:F:hiI:J::k:lLm:n::N::O:p::P::q::Q::st:Tvw:xyzZ::",
+	"aA::bBcC::d::De:E:f:F:hiI:J::k:lLm:n:N:O:p::P::q::Q::rR:sS:t:Tvw:xyzZ::",
 			    long_options, 0))
 	 != EOF)
     {
@@ -399,6 +403,10 @@ main (int argc, char **argv)
 	  output_style |= STYLE_SUMMARY_FILE;
 	  user_specified |= STYLE_SUMMARY_FILE;
 	  break;
+	case 'S':
+	  external_symbol_table = optarg;
+	  DBG (AOUTDEBUG, printf ("external-symbol-table: %s\n", optarg));
+	  break;
 	case 't':
 	  bb_table_length = atoi (optarg);
 	  if (bb_table_length < 0)
@@ -411,7 +419,7 @@ main (int argc, char **argv)
 	  break;
 	case 'v':
 	  /* This output is intended to follow the GNU standards document.  */
-	  printf (_("GNU gprof %s\n"), VERSION);
+	  printf (_("GNU gprof %s\n"), BFD_VERSION_STRING);
 	  printf (_("Based on BSD gprof, copyright 1983 Regents of the University of California.\n"));
 	  printf (_("\
 This program is free software.  This program has absolutely no warranty.\n"));
@@ -447,21 +455,32 @@ This program is free software.  This program has absolutely no warranty.\n"));
 	case OPTION_DEMANGLE:
 	  demangle = TRUE;
 	  if (optarg != NULL)
-	    style = optarg;
+	    {
+	      enum demangling_styles style;
+
+	      style = cplus_demangle_name_to_style (optarg);
+	      if (style == unknown_demangling)
+		{
+		  fprintf (stderr,
+			   _("%s: unknown demangling style `%s'\n"),
+			   whoami, optarg);
+		  xexit (1);
+		}
+
+	      cplus_demangle_set_style (style);
+	   }
 	  break;
 	case OPTION_NO_DEMANGLE:
 	  demangle = FALSE;
 	  break;
-	case OPTION_DEMANGLER:
-	  demangler = optarg;
+	case OPTION_INLINE_FILE_NAMES:
+	  inline_file_names = TRUE;
 	  break;
 	default:
 	  usage (stderr, 1);
 	}
     }
 
-  init_demangler (style, NULL, demangler);
-
   /* Don't allow both ordering options, they modify the arc data in-place.  */
   if ((user_specified & STYLE_FUNCTION_ORDER)
       && (user_specified & STYLE_FILE_ORDER))
@@ -505,7 +524,9 @@ This program is free software.  This program has absolutely no warranty.\n"));
     core_get_text_space (core_bfd);
 
   /* Create symbols from core image.  */
-  if (line_granularity)
+  if (external_symbol_table)
+    core_create_syms_from (external_symbol_table);
+  else if (line_granularity)
     core_create_line_syms ();
   else
     core_create_function_syms ();
@@ -537,7 +558,12 @@ This program is free software.  This program has absolutely no warranty.\n"));
   if (output_style == 0)
     {
       if (gmon_input & (INPUT_HISTOGRAM | INPUT_CALL_GRAPH))
-	output_style = STYLE_FLAT_PROFILE | STYLE_CALL_GRAPH;
+	{
+	  if (gmon_input & INPUT_HISTOGRAM)
+	    output_style |= STYLE_FLAT_PROFILE;
+	  if (gmon_input & INPUT_CALL_GRAPH)
+	    output_style |= STYLE_CALL_GRAPH;
+	}
       else
 	output_style = STYLE_EXEC_COUNTS;
 
@@ -586,7 +612,7 @@ This program is free software.  This program has absolutely no warranty.\n"));
   if (output_style & STYLE_FLAT_PROFILE)
     {
       /* Print the flat profile.  */
-      hist_print ();		
+      hist_print ();
     }
 
   if (cg && (output_style & STYLE_CALL_GRAPH))
@@ -594,20 +620,20 @@ This program is free software.  This program has absolutely no warranty.\n"));
       if (!bsd_style_output)
 	{
 	  /* Print the dynamic profile.  */
-	  cg_print (cg);	
+	  cg_print (cg);
 	}
       cg_print_index ();
     }
 
   if (output_style & STYLE_EXEC_COUNTS)
     print_exec_counts ();
-  
+
   if (output_style & STYLE_ANNOTATED_SOURCE)
     print_annotated_source ();
-  
+
   if (output_style & STYLE_FUNCTION_ORDER)
     cg_print_function_ordering ();
-  
+
   if (output_style & STYLE_FILE_ORDER)
     cg_print_file_ordering ();
 
