@@ -1,33 +1,35 @@
 /* i370.h -- Header file for S/390 opcode table
-   Copyright 1994, 1995, 1998, 1999 Free Software Foundation, Inc.
+   Copyright (C) 1994-2015 Free Software Foundation, Inc.
    PowerPC version written by Ian Lance Taylor, Cygnus Support
    Rewritten for i370 ESA/390 support, Linas Vepstas <linas@linas.org>
 
-This file is part of GDB, GAS, and the GNU binutils.
+   This file is part of GDB, GAS, and the GNU binutils.
 
-GDB, GAS, and the GNU binutils are free software; you can redistribute
-them and/or modify them under the terms of the GNU General Public
-License as published by the Free Software Foundation; either version
-1, or (at your option) any later version.
+   GDB, GAS, and the GNU binutils are free software; you can redistribute
+   them and/or modify them under the terms of the GNU General Public
+   License as published by the Free Software Foundation; either version 3,
+   or (at your option) any later version.
 
-GDB, GAS, and the GNU binutils are distributed in the hope that they
-will be useful, but WITHOUT ANY WARRANTY; without even the implied
-warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
-the GNU General Public License for more details.
+   GDB, GAS, and the GNU binutils are distributed in the hope that they
+   will be useful, but WITHOUT ANY WARRANTY; without even the implied
+   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
+   the GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this file; see the file COPYING.  If not, write to the Free
-Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   You should have received a copy of the GNU General Public License
+   along with this file; see the file COPYING3.  If not, write to the Free
+   Software Foundation, 51 Franklin Street - Fifth Floor, Boston,
+   MA 02110-1301, USA.  */
 
 #ifndef I370_H
 #define I370_H
 
 /* The opcode table is an array of struct i370_opcode.  */
-typedef union { 
-     unsigned int   i[2];
-     unsigned short s[4];
-     unsigned char  b[8];
-  }  i370_insn_t;
+typedef union
+{
+   unsigned int   i[2];
+   unsigned short s[4];
+   unsigned char  b[8];
+}  i370_insn_t;
 
 struct i370_opcode
 {
@@ -158,8 +160,8 @@ struct i370_operand
      string (the operand will be inserted in any case).  If the
      operand value is legal, *ERRMSG will be unchanged (most operands
      can accept any value).  */
-  i370_insn_t (*insert) PARAMS ((i370_insn_t instruction, long op,
-				   const char **errmsg));
+  i370_insn_t (*insert)
+    (i370_insn_t instruction, long op, const char **errmsg);
 
   /* Extraction function.  This is used by the disassembler.  To
      extract this operand type from an instruction, check this field.
@@ -178,13 +180,13 @@ struct i370_operand
      non-zero if this operand type can not actually be extracted from
      this operand (i.e., the instruction does not match).  If the
      operand is valid, *INVALID will not be changed.  */
-  long (*extract) PARAMS ((i370_insn_t instruction, int *invalid));
+  long (*extract) (i370_insn_t instruction, int *invalid);
 
-   /* One bit syntax flags.  */
-   unsigned long flags;
+  /* One bit syntax flags.  */
+  unsigned long flags;
 
-   /* name -- handy for debugging, otherwise pointless */
-   char * name; 
+  /* name -- handy for debugging, otherwise pointless */
+  char * name;
 };
 
 /* Elements in the table are retrieved by indexing with values from
@@ -197,19 +199,19 @@ extern const struct i370_operand i370_operands[];
 /* This operand should be wrapped in parentheses rather than
    separated from the previous by a comma.  This is used for S, RS and
    SS form instructions which want their operands to look like
-        reg,displacement(basereg) */
+   reg,displacement(basereg) */
 #define I370_OPERAND_SBASE (0x01)
 
 /* This operand is a base register.  It may or may not appear next
    to an index register, i.e. either of the two forms
-        reg,displacement(basereg)
-        reg,displacement(index,basereg) */
+   reg,displacement(basereg)
+   reg,displacement(index,basereg) */
 #define I370_OPERAND_BASE (0x02)
 
 /* This pair of operands should be wrapped in parentheses rather than
-    separated from the last by a comma.  This is used for the RX form
-    instructions which want their operands to look like
-        reg,displacement(index,basereg) */
+   separated from the last by a comma.  This is used for the RX form
+   instructions which want their operands to look like
+   reg,displacement(index,basereg) */
 #define I370_OPERAND_INDEX (0x04)
 
 /* This operand names a register.  The disassembler uses this to print
@@ -235,7 +237,7 @@ extern const struct i370_operand i370_operands[];
 #define I370_OPERAND_OPTIONAL (0x80)
 
 
-/* Define some misc macros.  We keep them with the operands table 
+/* Define some misc macros.  We keep them with the operands table
    for simplicity.  The macro table is an array of struct i370_macro.  */
 
 struct i370_macro
