@@ -1,5 +1,5 @@
 SCRIPT_NAME=m88kbcs
 OUTPUT_FORMAT="coff-m88kbcs"
-TEXT_START_ADDR=??
-TARGET_PAGE_SIZE=??
+TEXT_START_ADDR=0
+TARGET_PAGE_SIZE=128
 ARCH=m88k
